==> fotoalbum-0.33.tar.bz2: fotoalbum-0.33/src/base/blockingtask.cpp <==
/*
 * Copyright (C) 2011  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#include "blockingtask.h"
#include "../shared/thread.h"
#include "../shared/filetools.h"
#include "../shared/lib_conversions.h"
#include "../config.h"

#include <cstdlib>
#include <iostream>
#include <unistd.h>
#include <cstring>

#ifdef USE_INTEGRATED_JPEGTRAN
# include <jpeglib.h>
# include "../jpegtran/jpegtran_lib.h"
#endif

#ifdef USE_LIBGRAPHICSMAGICK
# include <magick/api.h>
#endif


void* WorkOnBlockingTasks(void* td)
{
  BlockTaskThread* btd = (BlockTaskThread*)(td);
  
  while (!btd->tasks.empty() && !btd->fCancel)
  {
    while (btd->fPause || btd->fPauseUser)
      usleep(100000);
    
    std::list<BlockingTask*>::iterator it = btd->tasks.begin();
    BlockingTask* bt = *it;
    if (bt->Exec())
      btd->nErrCnt++;
    btd->tasks.pop_front();
    delete bt;
    btd->nDone++;    
  }

  if (btd->fCancel)
  {
    while (!btd->tasks.empty())
    {
      std::list<BlockingTask*>::iterator it = btd->tasks.begin();
      BlockingTask* bt = *it;
      btd->tasks.pop_front();
      delete bt;
    }
    btd->nDone = btd->nSize;
  }
    
  SimpleThread::Exit();
  return((void*)(btd->nErrCnt));
}

BlockTaskThread::BlockTaskThread()
{
  nDone = 0;
  nSize = 0;
  fPause = false;
  fPauseUser = false;
}

void BlockTaskManager::Pause(bool fPause)
{
  threadData.fPause = fPause;
}

void BlockTaskManager::PauseUser(bool fPause)
{
  threadData.fPauseUser = fPause;
}

void BlockTaskManager::Add(BlockingTask* pTask)
{
  tasks.push_back(pTask);
}

bool BlockTaskManager::Status(int& nDone, int& nSize)
{
  nDone = threadData.nDone;
  nSize = threadData.nSize + tasks.size();
  return(nDone == nSize);
}

int BlockTaskManager::Step()
{
  if (threadData.nDone == threadData.nSize)
  {
    if (threadData.fCancel)
    {
      while (!tasks.empty())
      {
        std::list<BlockingTask*>::iterator it = tasks.begin();
        BlockingTask* bt = *it;
        tasks.pop_front();
        delete bt;
      }
      
      threadData.fCancel = false;
      threadData.nDone = 0;
      threadData.nSize = 0;
      return(-1);
    }
    else if (!tasks.empty() && !threadData.fPause && !threadData.fPauseUser)
    {
      int nRetCode = 0;
      if (threadData.nDone == 0)
      {
        nRetCode = 1;
      }
      
      std::cout << "void BlockTaskManager::Step() starting thread\n";
      // Lets start a new thread
      threadData.tasks.splice(threadData.tasks.begin(), tasks);
      threadData.nDone = 0;
      threadData.nSize = threadData.tasks.size();
      threadData.nErrCnt = 0;
      SimpleThread::Create(WorkOnBlockingTasks, (void*)&threadData);
      
      return(nRetCode);
    }
    else if (tasks.empty() && threadData.nDone > 0)
    {
      threadData.nDone = 0;
      threadData.nSize = 0;
      return(-1);
    }
  }  
  return(0);
}

BlockingTask_System::BlockingTask_System(std::string strCmd)
{
  this->strCmd = strCmd;
}

int BlockingTask_System::Exec()
{
  std::cout << "blocking plugin: " << strCmd << "\n";
  system(strCmd.c_str());
  return(0);
}

BlockingTask_Rotate::BlockingTask_Rotate(std::string strPath, int nDegrees)
{
  this->strPath  = strPath;
  this->nDegrees = -nDegrees;
  if (this->nDegrees < 0)
    this->nDegrees += 360;
}

int BlockingTask_Rotate::Exec()
{
  // todo: test if file is writeable
    
  bool fJPEG;
  {
    std::string suff = strU(FileTools::getSuffix(strPath));
    fJPEG = suff.compare(".JPEG") == 0 || suff.compare(".JPG") == 0;
  }
  
  if (fJPEG)
  {
#ifdef USE_INTEGRATED_JPEGTRAN
    // try internal jpegtran
    {
      const int nNumArgs = 9;
      int nRetCode;
      
      const char* args[nNumArgs];
      
      const char* fixedargs[] = {"jpegtran",  "-rotate", "-trim", "-copy", "all", "-outfile"};
      std::string alpha       = itoStr(nDegrees, ' ', 1);
      
      char* cstr_alpha = (char*)malloc(alpha.length()    +1);
      char* cstr_src   = (char*)malloc(strPath.length() +1);
      char* cstr_dst   = (char*)malloc(strPath.length() +1);
      
      if (cstr_alpha == 0 || cstr_src == 0 || cstr_dst == 0)
      {
        std::cerr << "Error rotating: malloc failed: " << alpha << " " << strPath << "\n";
        return(-1);
      }
      
      strcpy(cstr_alpha, alpha.c_str());
      strcpy(cstr_src,   strPath.c_str());
      strcpy(cstr_dst,   strPath.c_str());
      
      args[0] = fixedargs[0];
      args[1] = fixedargs[1];
      args[2] = cstr_alpha;
      args[3] = fixedargs[2];
      args[4] = fixedargs[3];
      args[5] = fixedargs[4];
      args[6] = fixedargs[5];
      args[7] = cstr_dst;
      args[8] = cstr_src;

      nRetCode = jpegtran_main_lib(nNumArgs, args);
      
      free(cstr_alpha);
      free(cstr_src);
      free(cstr_dst);
      
      if (nRetCode == 0)
      {
        return(0);
      }
      else
      {
        std::cerr << "error using internal jpegtran: " << strPath << "\n";
      }
    }
#endif
    
    // try external jpegtran
    {
      // create temp file
      std::string tempfile;
      int  tempfd = TemporaryTextFile::GetTempFD(tempfile);
      
      if (tempfd != -1)
      {
        close(tempfd);
        
        std::string call = "jpegtran -rotate "
          + itoStr(nDegrees, ' ', 1)
            + " -trim -copy all -outfile " 
          + tempfile 
          + " "
          + "\"" + strPath + "\"";
        
        if (system(call.c_str()) == 0)
        {
          FileTools::move(strPath, tempfile);
          return(0);
        }
        else
          std::cerr << "external jpegtran failed, command line: " << call << "\n";
      }
    }
  }
  
#ifdef USE_LIBGRAPHICSMAGICK
  {
    ExceptionInfo exception;
    Image* image;
    Image* rot_image;
    ImageInfo* image_info;
  
    GetExceptionInfo(&exception);
    image_info=CloneImageInfo((ImageInfo *) NULL);
    
    strcpy(image_info->filename, strPath.c_str());
    //std::cout << "Reading:" << image_info->filename << ", ";
    
    image=ReadImage(image_info, &exception);
    //std::cout << GetImageListLength(image) << " frames\n";
    // DescribeImage(image, stdout, 1);
    CatchException(&exception);
    if (image != 0)
    {
      rot_image=RotateImage(image, nDegrees, &exception);
      // DescribeImage(rot_image, stdout, 1);
      CatchException(&exception);
      if (rot_image)
      {
        strcpy(rot_image->filename, strPath.c_str());
        //std::cout << "Writing:" << rot_image->filename << ", " << image_info->quality << "\n";
        if (WriteImage(image_info, rot_image) != 0)
        {
          DestroyImage(image);
          DestroyImage(rot_image);
          DestroyImageInfo(image_info);
          DestroyExceptionInfo(&exception);
          return(0);
        }
        
        CatchException(&(rot_image->exception));
        std::cerr << "rotating with libgraphicsmagick failed(1), " << strPath << "\n";
      }
      // DescribeImage(rot_image, stdout, 1);
    }
    else
    {
      std::cerr << "rotating with libgraphicsmagick failed(2), " << strPath << "\n";
    }
  }
#endif
    
  // try convert (imagemagick)
  {
    // create temp file
    std::string tempfile;
    int  tempfd = TemporaryTextFile::GetTempFD(tempfile);

    if (tempfd != -1)
    {
      close(tempfd);
      
      std::string call = "convert \"" + strPath + "\" -rotate "
        + itoStr(nDegrees, ' ', 1) + " "
        + tempfile;

      if (system(call.c_str()) == 0)
      {
        FileTools::move(strPath, tempfile);
        return(0);
      }
      else
        std::cerr << "rotating with convert failed, " << call << "\n";
    }
  }
   
  // give up
  return(-1);
}

==> fotoalbum-0.104.tar.bz2: fotoalbum-0.104/src/base/blockingtask.cpp <==
/*
 * Copyright (C) 2011  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#include "blockingtask.h"
#include "../shared/thread.h"
#include "../shared/filetools.h"
#include "../shared/lib_conversions.h"
#include "../config.h"

#include <cstdlib>
#include <iostream>
#include <unistd.h>
#include <cstring>

#ifdef USE_INTEGRATED_JPEGTRAN
# include <jpeglib.h>
# include "../jpegtran/jpegtran_lib.h"
#endif

#ifdef USE_LIBGRAPHICSMAGICK
# include <magick/api.h>
#endif


void* WorkOnBlockingTasks(void* td)
{
  BlockTaskThread* btd = (BlockTaskThread*)(td);
  
  while (!btd->tasks.empty() && !btd->fCancel)
  {
    while (btd->fPause || btd->fPauseUser)
      usleep(100000);
    
    std::list<BlockingTask*>::iterator it = btd->tasks.begin();
    BlockingTask* bt = *it;
    if (bt->Exec())
      btd->nErrCnt++;
    btd->tasks.pop_front();
    delete bt;
    btd->nDone++;    
  }

  if (btd->fCancel)
  {
    while (!btd->tasks.empty())
    {
      std::list<BlockingTask*>::iterator it = btd->tasks.begin();
      BlockingTask* bt = *it;
      btd->tasks.pop_front();
      delete bt;
    }
    btd->nDone = btd->nSize;
  }
    
  SimpleThread::Exit();
  return((void*)(btd->nErrCnt));
}

BlockTaskThread::BlockTaskThread()
{
  nDone = 0;
  nSize = 0;
  fPause = false;
  fPauseUser = false;
}

void BlockTaskManager::Pause(bool fPause)
{
  threadData.fPause = fPause;
}

void BlockTaskManager::PauseUser(bool fPause)
{
  threadData.fPauseUser = fPause;
}

void BlockTaskManager::Add(BlockingTask* pTask)
{
  tasks.push_back(pTask);
}

bool BlockTaskManager::FooStatus(int& nDone, int& nSize)
{
  nDone = threadData.nDone;
  nSize = threadData.nSize + tasks.size();
  return(nDone == nSize);
}

int BlockTaskManager::Step()
{
  if (threadData.nDone == threadData.nSize)
  {
    if (threadData.fCancel)
    {
      while (!tasks.empty())
      {
        std::list<BlockingTask*>::iterator it = tasks.begin();
        BlockingTask* bt = *it;
        tasks.pop_front();
        delete bt;
      }
      
      threadData.fCancel = false;
      threadData.nDone = 0;
      threadData.nSize = 0;
      return(-1);
    }
    else if (!tasks.empty() && !threadData.fPause && !threadData.fPauseUser)
    {
      int nRetCode = 0;
      if (threadData.nDone == 0)
      {
        nRetCode = 1;
      }
      
      std::cout << "void BlockTaskManager::Step() starting thread\n";
      // Lets start a new thread
      threadData.tasks.splice(threadData.tasks.begin(), tasks);
      threadData.nDone = 0;
      threadData.nSize = threadData.tasks.size();
      threadData.nErrCnt = 0;
      SimpleThread::Create(WorkOnBlockingTasks, (void*)&threadData);
      
      return(nRetCode);
    }
    else if (tasks.empty() && threadData.nDone > 0)
    {
      threadData.nDone = 0;
      threadData.nSize = 0;
      return(-1);
    }
  }  
  return(0);
}

BlockingTask_System::BlockingTask_System(std::string strCmd)
{
  this->strCmd = strCmd;
}

int BlockingTask_System::Exec()
{
  std::cout << "blocking plugin: " << strCmd << "\n";
  system(strCmd.c_str());
  return(0);
}

BlockingTask_Rotate::BlockingTask_Rotate(std::string strPath, int nDegrees)
{
  this->strPath  = strPath;
  this->nDegrees = -nDegrees;
  if (this->nDegrees < 0)
    this->nDegrees += 360;
}

int BlockingTask_Rotate::Exec()
{
  // todo: test if file is writeable
    
  bool fJPEG;
  {
    std::string suff = strU(FileTools::getSuffix(strPath));
    fJPEG = suff.compare(".JPEG") == 0 || suff.compare(".JPG") == 0;
  }
  
  if (fJPEG)
  {
#ifdef USE_INTEGRATED_JPEGTRAN
    // try internal jpegtran
    {
      const int nNumArgs = 9;
      int nRetCode;
      
      const char* args[nNumArgs];
      
      const char* fixedargs[] = {"jpegtran",  "-rotate", "-trim", "-copy", "all", "-outfile"};
      std::string alpha       = itoStr(nDegrees, ' ', 1);
      
      char* cstr_alpha = (char*)malloc(alpha.length()    +1);
      char* cstr_src   = (char*)malloc(strPath.length() +1);
      char* cstr_dst   = (char*)malloc(strPath.length() +1);
      
      if (cstr_alpha == 0 || cstr_src == 0 || cstr_dst == 0)
      {
        std::cerr << "Error rotating: malloc failed: " << alpha << " " << strPath << "\n";
        return(-1);
      }
      
      strcpy(cstr_alpha, alpha.c_str());
      strcpy(cstr_src,   strPath.c_str());
      strcpy(cstr_dst,   strPath.c_str());
      
      args[0] = fixedargs[0];
      args[1] = fixedargs[1];
      args[2] = cstr_alpha;
      args[3] = fixedargs[2];
      args[4] = fixedargs[3];
      args[5] = fixedargs[4];
      args[6] = fixedargs[5];
      args[7] = cstr_dst;
      args[8] = cstr_src;

      nRetCode = jpegtran_main_lib(nNumArgs, args);
      
      free(cstr_alpha);
      free(cstr_src);
      free(cstr_dst);
      
      if (nRetCode == 0)
      {
        return(0);
      }
      else
      {
        std::cerr << "error using internal jpegtran: " << strPath << "\n";
      }
    }
#endif
    
    // try external jpegtran
    {
      // create temp file
      std::string tempfile;
      int  tempfd = TemporaryTextFile::GetTempFD(tempfile);
      
      if (tempfd != -1)
      {
        close(tempfd);
        
        std::string call = "jpegtran -rotate "
          + itoStr(nDegrees, ' ', 1)
            + " -trim -copy all -outfile " 
          + tempfile 
          + " "
          + "\"" + strPath + "\"";
        
        if (system(call.c_str()) == 0)
        {
          FileTools::move(strPath, tempfile);
          return(0);
        }
        else
          std::cerr << "external jpegtran failed, command line: " << call << "\n";
      }
    }
  }
  
#ifdef USE_LIBGRAPHICSMAGICK
  {
    ExceptionInfo exception;
    Image* image;
    Image* rot_image;
    ImageInfo* image_info;
  
    GetExceptionInfo(&exception);
    image_info=CloneImageInfo((ImageInfo *) NULL);
    
    strcpy(image_info->filename, strPath.c_str());
    //std::cout << "Reading:" << image_info->filename << ", ";
    
    image=ReadImage(image_info, &exception);
    //std::cout << GetImageListLength(image) << " frames\n";
    // DescribeImage(image, stdout, 1);
    CatchException(&exception);
    if (image != 0)
    {
      rot_image=RotateImage(image, nDegrees, &exception);
      // DescribeImage(rot_image, stdout, 1);
      CatchException(&exception);
      if (rot_image)
      {
        strcpy(rot_image->filename, strPath.c_str());
        //std::cout << "Writing:" << rot_image->filename << ", " << image_info->quality << "\n";
        if (WriteImage(image_info, rot_image) != 0)
        {
          DestroyImage(image);
          DestroyImage(rot_image);
          DestroyImageInfo(image_info);
          DestroyExceptionInfo(&exception);
          return(0);
        }
        
        CatchException(&(rot_image->exception));
        std::cerr << "rotating with libgraphicsmagick failed(1), " << strPath << "\n";
      }
      // DescribeImage(rot_image, stdout, 1);
    }
    else
    {
      std::cerr << "rotating with libgraphicsmagick failed(2), " << strPath << "\n";
    }
  }
#endif
    
  // try convert (imagemagick)
  {
    // create temp file
    std::string tempfile;
    int  tempfd = TemporaryTextFile::GetTempFD(tempfile);

    if (tempfd != -1)
    {
      close(tempfd);
      
      std::string call = "convert \"" + strPath + "\" -rotate "
        + itoStr(nDegrees, ' ', 1) + " "
        + tempfile;

      if (system(call.c_str()) == 0)
      {
        FileTools::move(strPath, tempfile);
        return(0);
      }
      else
        std::cerr << "rotating with convert failed, " << call << "\n";
    }
  }
   
  // give up
  return(-1);
}
